package com.example.nto.dto.mappers.employee;

import com.example.nto.domain.entity.*;
import com.example.nto.dto.entity.employee.EmployeeDTO;
import com.example.nto.utils.Utils;
import lombok.experimental.UtilityClass;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

@UtilityClass
public class EmployeeMapper {
    public static EmployeeDTO convertToDTO(Employee employee) {
        EmployeeDTO employeeDTO = new EmployeeDTO();

        employeeDTO.setId(employee.getId());
        employeeDTO.setName(employee.getName());
        employeeDTO.setSurname(employee.getSurname());
        employeeDTO.setPatronymic(employee.getPatronymic());
        employeeDTO.setAboutMe(employeeDTO.getAboutMe());
        employeeDTO.setTelephone(employee.getTelephone());
        employeeDTO.setEmail(employee.getEmail());

        employeeDTO.setOfficeId(employee.getOffice().getId());
        employeeDTO.setOfficeName(employee.getOffice().getName());
        employeeDTO.setOfficeImageUrl(employee.getOffice().getLinkLogo());
        employeeDTO.setPositionName(employee.getPosition().getName());
        employeeDTO.setRole(employee.getRole().getRoleName());
        employeeDTO.setProfileImageUrl(employee.getProfileImageUrl());
        employeeDTO.setBlocked(employee.isBlocked());

        List<Visit> visitsLast30Days = Utils.filterDateLast30Days(employee.getVisits());

        boolean visitStatus = false;
        String startVisitDateTime = null;
        String currentOfficeName = null;
        List<Long> visitsIdLast30Days = new ArrayList<>(visitsLast30Days.size());
        List<List<LocalDateTime>> periods = new ArrayList<>(visitsLast30Days.size());

        for (Visit visit : visitsLast30Days) {
            if (!visit.isFinished()) {
                if (visitStatus) throw new IllegalStateException("У работника два посещения одновременно!");

                visitStatus = true;
                startVisitDateTime = visit.getStartVisit().toString();
                currentOfficeName = visit.getStartTerminal().getOffice().getName();
            }
            visitsIdLast30Days.add(visit.getId());

            if (visit.getStartVisit() != null && visit.getEndVisit() != null) {
                periods.add(List.of(visit.getStartVisit(), visit.getEndVisit()));
            }
        }

        employeeDTO.setVisitStatus(visitStatus);
        employeeDTO.setStartVisitDateTime(startVisitDateTime);
        employeeDTO.setCurrentOfficeName(currentOfficeName);
        employeeDTO.setVisitsIdLast30Days(visitsIdLast30Days);
        employeeDTO.setTotalTimeVisitsLast30Days(Utils.periods(periods));
        employeeDTO.setCreateAt(employee.getCreatedAt().toString());

        return employeeDTO;
    }

    public static Employee convertFromDTO(EmployeeDTO employeeDTO, Office office, Position position, Role role) {
        Employee employee = new Employee();

        employee.setName(employeeDTO.getName());
        employee.setSurname(employeeDTO.getSurname());
        employee.setPatronymic(employeeDTO.getPatronymic());
        employeeDTO.setAboutMe(employee.getAboutMe());
        employee.setTelephone(employeeDTO.getTelephone());
        employee.setEmail(employeeDTO.getEmail());
        employee.setOffice(office);
        employee.setPosition(position);
        employee.setRole(role);
        employee.setProfileImageUrl(employeeDTO.getProfileImageUrl());
        employee.setBlocked(employeeDTO.isBlocked());

        return employee;
    }
}

